﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Data;
using System.Collections;

using FogCreek.FogBugz.Plugins;
using FogCreek.Plugins.BugField;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Entity;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz;
using FogCreek.FogBugz.Globalization;
using FogCreek.FogBugz.UI;
using FogCreek.FogBugz.UI.Dialog;
using FogCreek.FogBugz.Database;


namespace CRM_Plugin
{
    public class CRM_Plugin : CBugFieldPlugin, IPluginBugJoin, IPluginBugCommit, IPluginBugDisplay,
                              IPluginDatabase, IPluginStaticFile, IPluginSearchAxis
    {
		private const string PLUGIN_ID = "thisshouldmatchyourassemblyattribute";
        private const string CUST_ID_DISPLAY_NAME = "Customer ID";
        private const string CUST_ID_DESCRIPTION = "The Customer ID in the CRM which the case is associated with.";
        private const string CUST_ID_COL_NAME = "ixCustomer";
        private const string CUST_ID_SIMPLE_NAME = "customerid"; // also the search axis
        private const int CUST_ID_BUGFIELD_ID = 1;
        private const int DATABASE_SCHEMA_VERSION = 1;

        private const string CRM_URL = "http://crm.example.com/default.aspx";

        private readonly string VARIOUS_TEXT = CLocalizer.Current.FB_VARIOUS;
        private string sBugFieldTableName;

        public CRM_Plugin(CPluginApi api)
            : base(api)
        {
            // this is hard-coded in BugField's code, but not publicly accessible
            sBugFieldTableName = api.Database.PluginTableName("BugField");
        }

        #region IPluginDatabase Members

        public override int DatabaseSchemaVersion()
        {
            return DATABASE_SCHEMA_VERSION; // initial schema: one joined column: ixCustomer
        }

        #endregion

        #region CBugFieldPlugin Members

        public override CBugField[] BugFields()
        {

            CNumberBugField field = new CNumberBugField(api,
                                                  CUST_ID_DISPLAY_NAME,
                                                  CUST_ID_DESCRIPTION,
                                                  PLUGIN_ID,
                                                  CUST_ID_COL_NAME,
                                                  CUST_ID_BUGFIELD_ID,
                                                  CUST_ID_COL_NAME + "_comparator");

            field.fLeft = true;
            field.sSimpleName = CUST_ID_SIMPLE_NAME;
            field.nDisplayTypeClose = DisplayType.ReadOnly;
            field.nDisplayTypeCreate = DisplayType.Editable;
            field.nDisplayTypeEdit = DisplayType.Editable;
            field.nDisplayTypePublic = DisplayType.Hidden;
            field.nDisplayTypeResolve = DisplayType.Editable;

            return new CBugField[] { field };
        }

		// overriding BugDisplayViewLeft to display a link instead of just the field value in view mode
        public override CBugDisplayDialogItem[] BugDisplayViewLeft(CBug[] rgbug, bool fPublic)
        {
            CBugDisplayDialogItem[] dialogItems = base.BugDisplayViewLeft(rgbug, fPublic);
            int i;
            for (i = 0; i < dialogItems.Length; i++)
            {
                // if it's the customer ID dialogitem, we want to tweak it
                if (dialogItems[i].sLabel == CUST_ID_DISPLAY_NAME)
                {
                    // I don't think the icon is used in view mode but I might as well set it
                    dialogItems[i].sIconURL = api.Url.PluginStaticFileUrl("customer_icon.gif");
                    string sContent = dialogItems[i].sContent;
                    int ixCustomer = -1;
                    // if it's not "various" and the ixCustomer is defined (positive int),
                    // replace the default sContent from BugField with the value and the links
                    if (sContent.IndexOf(VARIOUS_TEXT) < 0)
                    {
                        if (rgbug.Length > 0 && (ixCustomer = Convert.ToInt32(rgbug[0].GetPluginField(PLUGIN_ID, CUST_ID_COL_NAME))) > 0)
                        {
                            dialogItems[i].sContent = string.Format(
                                @"{1}<br />
                                 <a href=""{0}?pg=pgCustomer&ixCustomer={1}"" target=""_blank""
                                    title=""View customer {1} in CRM"">View in CRM</a><br />
                                 <a href=""{2}?pg=pgList&searchFor=customerid:{1}&search=1""
                                    title=""Find all FogBugz cases for customer {1}"">Search FogBugz</a>",
                                CRM_URL,
                                ixCustomer.ToString(),
                                api.Url.BaseUrl());
                        }
                        else
                        {
                            dialogItems[i] = null;
                        }
                    }
                }
            }

            return dialogItems;
        }

		// override bugfield's BugDisplayEditLeft to add our custom icon
        public override CBugDisplayDialogItem[] BugDisplayEditLeft(CBug[] rgbug, BugEditMode nMode, bool fPublic)
        {
            CBugDisplayDialogItem[] dialogItems = base.BugDisplayEditLeft(rgbug, nMode, fPublic);
            for (int i = 0; i < dialogItems.Length; i++)
            {
                // if it's the customer ID dialogitem, add the icon
                if (dialogItems[i].sLabel == CUST_ID_DISPLAY_NAME)
                {
                    dialogItems[i].sIconURL = api.Url.PluginStaticFileUrl("customer_icon.gif");
                }
            }
            return dialogItems;
        }

		// bugfield adds a search axis for us, but we're overriding to allow for searching for "*"
        public override CSelectQuery SearchAxisQuery(string sAxis, SearchType nType, string sValue)
        {
            CSelectQuery select = base.SearchAxisQuery(sAxis, nType, sValue);
            // allow searching for "*", which returns all cases which have customer ID defined
            // this automatically adds searching for -customerid:"*" to get cases w/out an ID
            if (nType == SearchType.Bug &&
                sAxis == CUST_ID_SIMPLE_NAME &&
                sValue == "*")
            {
                select = api.Database.NewSelectQuery("Bug");
                select.AddSelect("Bug.ixBug");
                select.AddLeftJoin(sBugFieldTableName,
                                   string.Format("Bug.ixBug = {0}.ixBug",
                                                 sBugFieldTableName));
                select.AddWhere(string.Format("{0}.{1} <> 0 AND {0}.{1} IS NOT NULL",
                                                  sBugFieldTableName,
                                                  CUST_ID_COL_NAME));
            }
            return select;
        }

		// override bugfield's BugCommitBefore to add extra validation beyond "must be a number"
        public override void BugCommitBefore(CBug bug, BugAction nBugAction, CBugEvent bugevent, bool fPublic)
        {
            if (api.Request[api.AddPluginPrefix(CUST_ID_COL_NAME)] != null)
            {
                string sInput = HttpUtility.HtmlEncode(api.Request[api.AddPluginPrefix(CUST_ID_COL_NAME)].ToString().Trim());
                if (sInput != VARIOUS_TEXT)
                {
                    try
                    {
                        // ixCustomer must be greater than 0
                        if (sInput != "" && Convert.ToInt32(sInput) < 1)
                            throw new System.FormatException();
                        else
                            base.BugCommitBefore(bug, nBugAction, bugevent, fPublic);
                    }
                    catch (System.FormatException)
                    {
                        DisplayCustomerIDEditError(sInput);
                    }
                    catch (System.OverflowException)
                    {
                        DisplayCustomerIDEditError(sInput);
                    }
                }
            }
            return;
        }

        #endregion

        #region IPluginStaticFile Members

        public void StaticFileHeaders(string sFilename)
        {
        }

        public PermissionLevel StaticFileVisibility(string sFilename)
        {
            return PermissionLevel.Normal;
        }

        #endregion

        private void DisplayCustomerIDEditError(string sInput)
        {
            api.Notifications.AddError(
                            string.Format("\"{0}\" is not a valid value for {1}.",
                                          sInput,
                                          CUST_ID_DISPLAY_NAME),
                            string.Format("The {0} field only accepts positive integers, so we could " +
                                              "not record the value you entered, \"{1}\".",
                                          CUST_ID_DISPLAY_NAME,
                                          sInput),
                            false);
        }
    }
}
